/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNdU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOdUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICdULAR PdURPOSE.  See the GNdU General Public License
    for more details.

    You should have received a copy of the GNdU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 dUSA

Application
    plasticStressedFoam

Description
    Steady-state segregated finite-volume solver of linear-elastic + perfect-plastic,
    small-strain deformation of a solid body.

    Simple elasto-plastic structural analysis code.
    Solves for the displacement vector field dU, also generating the
    stress tensor field sigma and distinguishing the occurance of plasticity.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "Switch.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "readMechanicalProperties.H"
#   include "createFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nCalculating displacement field\n" << endl;
 
    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Iteration: " << runTime.timeName() << nl << endl;

#       include "readPlasticStressedFoamControls.H"

        int iCorr = 0;
        scalar initialResidual = 0;

        do
        {
            volTensorField graddU = fvc::grad(dU);

            fvVectorMatrix dUEqn
            (
                fvm::laplacian(2*mu + lambda, dU, "laplacian(DdU,dU)")

             ==

              - fvc::div
                (
                    mu*graddU.T() + lambda*(I*tr(graddU)) - (mu + lambda)*graddU,
                    "div(sigmaExp)"
                )

              + fvc::div
                (
                    2.0*mu*deps_p + lambda*I*tr(deps_p),
                    "div(sigmaP)"
                )
            );

            //dUEqn.setComponentReference(1, 0, vector::X, 0);
            //dUEqn.setComponentReference(1, 0, vector::Z, 0);

            initialResidual = dUEqn.solve().initialResidual();

        } while (initialResidual > convergenceTolerance && ++iCorr < nCorr);

        U += dU;

#       include "calculateStress.H"

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
